import 'dart:convert';
import 'dart:developer';

import 'package:dio/dio.dart';

import 'exceptions.dart';

enum QueryMethod { getMethod, postMethod, putMethod, deleteMethod }

abstract class QUERY<T> extends DioExceptions {
  String get url => host + endPoint;

  String get endPoint;

  QueryMethod get queryMethod;

  String host = "";
  Map<String, dynamic>? get queryParameters => {};
  Map<String, dynamic>? get header => {};
  Map<String, dynamic> get auth => {};

  // String? get authToken => null;

  Future<T?> onSuccess(
      {required dynamic body, Function(String error)? onDataError});

  // Function(String error)? onDataError;

  final int connectTimeout = 15000;
  final int receiveTimeout = 15000;
  final int sendTimeout = 15000;

  result({
    required Response<dynamic> result,
    required Function(dynamic data)? onSuccess,
  }) async {
    await onSuccess?.call(result.data);
  }

  Dio common() {
    Map<String, dynamic> header = {
      "Content-Type": "application/json",
    };
    if (this.header != null) {
      header.addAll(this.header!);
    }
    header.addAll(auth);
    return Dio(
      BaseOptions(
        headers: header,
        connectTimeout: connectTimeout,
        receiveTimeout: receiveTimeout,
        sendTimeout: sendTimeout,
      ),
    );
  }

  String get api => endPoint
      .replaceAll("/api/", "")
      .replaceAll("https://", "")
      .replaceAll("http://", "")
      .replaceAll("/", "-");

  dynamic get body async => {};

  Future request({
    required Function(T? data)? onSuccess,
    Function(String error)? onError,
  }) async {
    log("api: $endPoint");
    log("queryParameters: $queryParameters");
    log("header: $header");
    log("body: $body");
    switch (queryMethod) {
      case QueryMethod.getMethod:
        await _get(onSuccess: onSuccess, onError: onError);
        break;
      case QueryMethod.postMethod:
        await _post(onSuccess: onSuccess, onError: onError);
        break;
      case QueryMethod.putMethod:
        await _put(onSuccess: onSuccess, onError: onError);
        break;
      case QueryMethod.deleteMethod:
        await _delete(onSuccess: onSuccess, onError: onError);
        break;
      default:
    }
  }

  Future _get({
    required Function(T? data)? onSuccess,
    Function(String error)? onError,
  }) async {
    try {
      Response response = await common().get(
        url,
        queryParameters: queryParameters,
      );
      await result(
        result: response,
        onSuccess: (body) async {
          log(jsonEncode(body));
          var _onSuccess = await this.onSuccess(
            body: body,
            onDataError: (error) => onError?.call(error),
          );
          await onSuccess?.call(_onSuccess);
        },
      );
    } on DioError catch (error) {
      log(endPoint);
      log(error.toString());
      onError?.call(fromDioError(error));
    }
  }

  Future _post({
    required Function(T? data)? onSuccess,
    Function(String error)? onError,
  }) async {
    try {
      final response = await common().post(
        url,
        queryParameters: queryParameters,
        data: await body ?? {},
      );
      await result(
          result: response,
          onSuccess: (body) async {
            log(jsonEncode(body));
            var _onSuccess = await this.onSuccess(
              body: body,
              onDataError: (error) => onError?.call(error),
            );
            await onSuccess?.call(_onSuccess);
          });
    } on DioError catch (error) {
      log(endPoint);
      log(error.message);
      onError?.call(fromDioError(error));
    }
  }

  Future _put({
    required Function(T? data)? onSuccess,
    Function(String error)? onError,
  }) async {
    try {
      final response = await common().put(
        url,
        queryParameters: queryParameters,
        data: await body ?? {},
      );
      await result(
          result: response,
          onSuccess: (body) async {
            log(jsonEncode(body));
            var _onSuccess = await this.onSuccess(
              body: body,
              onDataError: (error) => onError?.call(error),
            );
            await onSuccess?.call(_onSuccess);
          });
    } on DioError catch (error) {
      log(endPoint);
      log(error.toString());
      onError?.call(fromDioError(error));
    }
  }

  Future _delete({
    required Function(T? data)? onSuccess,
    Function(String error)? onError,
  }) async {
    try {
      final response = await common().delete(
        url,
        queryParameters: queryParameters,
        data: await body ?? {},
      );
      await result(
          result: response,
          onSuccess: (body) async {
            log(jsonEncode(body));
            var _onSuccess = await this.onSuccess(
              body: body,
              onDataError: (error) => onError?.call(error),
            );
            await onSuccess?.call(_onSuccess);
          });
    } on DioError catch (error) {
      log(endPoint);
      log(error.toString());
      onError?.call(fromDioError(error));
    }
  }
}
